
/*****************************************************************************
  BH1749NUC.ino

 Copyright (c) 2022 ROHM Co.,Ltd.

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
******************************************************************************/
#include <Arduino.h>
#include <HardwareSerial.h>
#include <Wire.h>
#include <BH1749NUC.h>

#define SYSTEM_BAUDRATE (115200)
#define ERROR_WAIT      (1000)
#define WAIT_TMT_MAX    (350)

#define BH1749_RED      (0)
#define BH1749_GREEN    (1)
#define BH1749_BLUE     (2)
#define BH1749_IR       (3)
#define BH1749_GREEN2   (4)

#define ERROR_POINT     (30)

BH1749NUC bh1749(BH1749NUC_ADDR_HIGH);
void error_func(int32_t result);

void setup()
{
    int32_t result;
    float32 version;

    (void)Serial.begin(SYSTEM_BAUDRATE);
    while (!Serial){
    }

    (void)Serial.println("BH1749NUC Sample Code Version : 1.1");

    version = bh1749.get_version();
    (void)Serial.print("BH1749NUC Driver Version : ");
    (void)Serial.println(version, 1);

    Wire.begin();

    result = bh1749.init();
    if (result == BH1749NUC_COMM_OK) {
        (void)bh1749.start();
    } else {
        error_func(result);
    }

    delay(WAIT_TMT_MAX);

    return;
}

void loop()
{
    int32_t  result;
    uint16_t color[BH1749NUC_COLOR_NUMBER];

    result = bh1749.get_val(color);
    if (result == BH1749NUC_COMM_OK) {
        (void)Serial.print("Red:Green:Blue:IR:Green2 = ");
        (void)Serial.print(color[BH1749_RED]);
        (void)Serial.print(",");
        (void)Serial.print(color[BH1749_GREEN]);
        (void)Serial.print(",");
        (void)Serial.print(color[BH1749_BLUE]);
        (void)Serial.print(",");
        (void)Serial.print(color[BH1749_IR]);
        (void)Serial.print(",");
        (void)Serial.print(color[BH1749_GREEN2]);
        (void)Serial.println();
    } else {
        (void)Serial.println("Error : Can't get sensor data!");
        error_func(result);
    }

    delay(WAIT_TMT_MAX);

    return;
}

void error_func(int32_t result)
{
    uint8_t cnt = 0;

    switch (result) {
        case BH1749NUC_COMM_ERROR:
            (void)Serial.println("Communication Error.");
            break;

        case BH1749NUC_ID_ERROR:
            (void)Serial.println("ID Error.");
            break;

        default:
            (void)Serial.println("Unknown Error.");
            break;
    }
    
    (void)Serial.println("Check System and Driver Parameter.");
    while (1) {
        (void)Serial.print(".");
        if (cnt < ERROR_POINT) {
            cnt++;
        } else {
            cnt = 0;
            (void)Serial.println();
        }
        delay(ERROR_WAIT);
    }

    return;
}
